local addonInfo, privateVars = ...

---------- init namespace ---------

if not EnKai then EnKai = {} end
if not EnKai.inventory then EnKai.inventory = {} end

local data        = privateVars.data

---------- make global functions local ---------

local _iInspectItemList           = Inspect.Item.List
local _iInspectItemDetail         = Inspect.Item.Detail
local _iUtilityItemSlotInventory  = Utility.Item.Slot.Inventory
local _iUtilityItemSlotQuest      = Utility.Item.Slot.Quest

---------- init local variables ---------

local _invManager = false

---------- local function block ---------

local function _fctGetInventory ()
  
  EnKaiInv[data.playerDetails.name] = { invDB = {}, invSlots = {} }
  
  local slots = { _iInspectItemList(_iUtilityItemSlotInventory()), 
                  _iInspectItemList(Utility.Item.Slot.Bank()), 
                  _iInspectItemList(Utility.Item.Slot.Vault()), 
                  _iInspectItemList(_iUtilityItemSlotQuest()) }
  
  local lu = {}
  
  for idx = 1, #slots, 1 do
    for slot, key in pairs(slots[idx]) do
      if key ~= nil then lu[slot] = true end
    end
  end
      
  local itemList = _iInspectItemDetail(lu)

  for key, v in pairs(itemList) do
    if v.stack == nil then v.stack = 1 end
  
    EnKaiInv[data.playerDetails.name].invSlots[key] = { id = v.type, stack = v.stack }    
  
    if EnKaiInv[data.playerDetails.name].invDB[v.type] == nil then
      EnKaiInv[data.playerDetails.name].invDB[v.type] = v.stack
    else
      EnKaiInv[data.playerDetails.name].invDB[v.type] = EnKaiInv[data.playerDetails.name].invDB[v.type] + v.stack
    end
  end   
  
end

local function _fctProcessUpdate (_, updates)

  if data.playerDetails == nil then return end

  if EnKaiInv[data.playerDetails.name] == nil then _fctGetInventory() end
  
  local invSlots = EnKaiInv[data.playerDetails.name].invSlots
  local invDB = EnKaiInv[data.playerDetails.name].invDB
  
  local updatedKeys = {}
  
  for slot, key in pairs(updates) do
    if EnKai.strings.startsWith(slot, 'sg') == false then
  
      if key == "nil" then key = false end
    
      if invSlots[slot] ~= nil then
        updatedKeys[invSlots[slot].id] = -invSlots[slot].stack
        invDB[invSlots[slot].id] = invDB[invSlots[slot].id] - invSlots[slot].stack
      end
      
      if key == false then
        invSlots[slot] = nil
      else
        local details = _iInspectItemDetail(slot)
        if details ~= nil then          
          
          if details.stack == nil then details.stack = 1 end
          invSlots[slot] = { id = details.type, stack = details.stack}
          
          updatedKeys[details.type] = details.stack
          
          if invDB[details.type] == nil then
            invDB[details.type] = details.stack
          else
            invDB[details.type] = invDB[details.type] + details.stack
          end
        end
      end
    end
  end
  
  EnKai.eventHandlers["EnKai.InventoryManager"]["Update"](updatedKeys)

end

---------- library public function block ---------

function EnKai.inventory.init ()

	if _invManager == true then return end
	
	if EnKai.internal.checkEvents ("EnKai.InventoryManager", true) == false then return nil end
	
	if EnKaiInv == nil then EnKaiInv = {} end
	
	Command.Event.Attach(Event.Item.Slot, _fctProcessUpdate, "EnKai.inventory.Item.Slot")
	Command.Event.Attach(Event.Item.Update, _fctProcessUpdate, "EnKai.inventory.Item.Update")
	
	EnKai.eventHandlers["EnKai.InventoryManager"]["Update"], EnKai.events["EnKai.InventoryManager"]["Update"] = Utility.Event.Create(addonInfo.identifier, "EnKai.InventoryManagerUpdate")
	
	_invManager = true

end

function EnKai.inventory.queryByKey (key)

	if _invManager == false then 
		EnKai.tools.error.display ("EnKai", "Inventory manager not initialzed", 1)
		return
	end

	if EnKaiInv[data.playerDetails.name] == nil then _fctGetInventory() end

	if EnKaiInv[data.playerDetails.name].invDB[key] ~= nil then return EnKaiInv[data.playerDetails.name].invDB[key] end
	
	if string.find(key, ',') ~= nil then	
		local details = _iInspectItemDetail(key)
		if EnKaiInv[data.playerDetails.name].invDB[details.type] ~= nil then return EnKaiInv[data.playerDetails.name].invDB[details.type] end
	end
	
	return 0

end

function EnKai.inventory.getAvailableSlots()

	local availSlots = {}
	local allSlots = nil
	
	local slots = _iInspectItemList(_iUtilityItemSlotInventory())
	local initOk = false
	
	for slot, details in pairs (slots) do
		if EnKai.strings.startsWith(slot, "sibg.") == false then initOk = true end 		
		if details == false then table.insert (availSlots, slot) end
	end
	
	if initOk == false then return false end
	
	return availSlots

end

function EnKai.inventory.getQuestItems ()

	local slots = _iInspectItemList(_iUtilityItemSlotQuest())
	local lu = {}
	
	for slot, key in pairs(slots) do
		if key ~= nil then lu[slot] = true end
	end
	
	return _iInspectItemDetail(lu)		

end

 function EnKai.inventory.getQuestItemSlot (type)
 
 	local info = EnKai.inventory.getQuestItems ()
 	
 	for slot, details in pairs(info) do
 		if details.type == type then
 			return slot
 		end
 	end
 
 	return nil
 
 end 